using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Collections;

namespace sharpFtpLibrary
{
    public class ftp
    {
        #region "varibili"

        /// <summary>
        /// Host FTP
        /// </summary>
        private string _Host;
        /// <summary>
        /// Username
        /// </summary>
        private string _UserName;
        /// <summary>
        /// Password
        /// </summary>
        private string _Password;

        /// <summary>
        /// Richiesta FTP
        /// </summary>
        FtpWebRequest _FtpRequest;
        /// <summary>
        /// Risposta ftp
        /// </summary>
        FtpWebResponse _FtpResponse;

        /// <summary>
        /// Flag SSL
        /// </summary>
        private bool _UseSSL = false;

        #endregion

        #region "Propriet"

        /// <summary>
        /// FTP Host URI
        /// </summary>
        public string Host
        {
            get
            {
                return _Host;
            }
            set
            {
                _Host = value;
            }
        }

        /// <summary>
        /// FTP User Name
        /// </summary>
        public string UserName 
        {
            get 
            {
                return _UserName;
            }
            set 
            {
                _UserName = value;
            }
        }

        /// <summary>
        /// FTP Password
        /// </summary>
        public string Password 
        {
            get 
            {
                return _Password;
            }
            set 
            {
                _Password = value;
            }
        }

        /// <summary>
        /// Flag utilizzo SSL
        /// </summary>
        public bool UseSSL 
        {
            get 
            {
                return _UseSSL;
            }
            set 
            {
                _UseSSL = value;
            }
        }

        #endregion

        public ftp()
        {
            
        }

        public string ListDirectory(string path)
        {
            if (path == null || path == "")
            {
                path = "/";
            }

            _FtpRequest = (FtpWebRequest)WebRequest.Create("ftp://" + _Host + path);
                        
            _FtpRequest.Credentials = new NetworkCredential(_UserName, _Password);
            _FtpRequest.Method = WebRequestMethods.Ftp.ListDirectoryDetails;

            _FtpRequest.EnableSsl = _UseSSL;

            _FtpResponse = (FtpWebResponse) _FtpRequest.GetResponse();

            string liststring = "";
            try
            {
                StreamReader sr = new StreamReader(_FtpResponse.GetResponseStream(), System.Text.Encoding.ASCII);
                liststring = sr.ReadToEnd();
                sr.Close();
            }
            catch (WebException e)
            {
                Console.WriteLine(((FtpWebResponse)e.Response).StatusDescription);
            }

            _FtpResponse.Close();            

            return liststring;
        }

        public void DownloadFile(string Path, string LocalPath, string Name) 
        {
            string _fileName = LocalPath + "\\" + Name;

            FileInfo _File = new FileInfo(_fileName);
            FileStream _FileStream;

            _FtpRequest = (FtpWebRequest)WebRequest.Create("ftp://" + _Host + Path + Name);

            _FtpRequest.Credentials = new NetworkCredential(_UserName, _Password);
            _FtpRequest.Method = WebRequestMethods.Ftp.DownloadFile;

            _FtpRequest.EnableSsl = _UseSSL;
             
            _FileStream = new FileStream(_File.FullName, FileMode.Create, FileAccess.Write);

            _FtpResponse = (FtpWebResponse)_FtpRequest.GetResponse();

            Stream _ResponseStream = _FtpResponse.GetResponseStream();


            byte[] buffer = new byte[1024];
            int bytesRead = _ResponseStream.Read(buffer, 0, 1024);

            while (bytesRead != 0)
            {
                _FileStream.Write(buffer, 0, bytesRead);
                bytesRead = _ResponseStream.Read(buffer, 0, 1024);
            }
            
            _FileStream.Close();
            _ResponseStream.Close();
            
            
        }

        public void UploadFile(string Path, string LocalPath, string Name)
        {
            string _fileName = LocalPath + "\\" + Name;
            FileInfo _File = new FileInfo(_fileName);
            
            _FtpRequest = (FtpWebRequest)WebRequest.Create("ftp://" + _Host + Path + Name);
            _FtpRequest.Credentials = new NetworkCredential(_UserName, _Password);
            _FtpRequest.Method = WebRequestMethods.Ftp.UploadFile;
            _FtpRequest.EnableSsl = _UseSSL;

            byte[] _fileContents = new byte[_File.Length];

            FileStream fr = _File.OpenRead();
           
            fr.Read(_fileContents, 0, Convert.ToInt32(_File.Length));
            
            fr.Close();

            Stream writer = _FtpRequest.GetRequestStream();
            
            writer.Write(_fileContents, 0, _fileContents.Length);
            writer.Close();

        }

        public void DeleteFile(string Path, string Name)
        {
            _FtpRequest = (FtpWebRequest)WebRequest.Create("ftp://" + _Host + Path + Name);
            _FtpRequest.Credentials = new NetworkCredential(_UserName, _Password);
            _FtpRequest.Method = WebRequestMethods.Ftp.DeleteFile;
            _FtpRequest.EnableSsl = _UseSSL;

            _FtpResponse = (FtpWebResponse)_FtpRequest.GetResponse();
            Stream _ResponseStream = _FtpResponse.GetResponseStream();
            _ResponseStream.Close();
        }


        public void ResumeDownloadFile(string Path, string LocalPath, string Name)
        {
            string _fileName = LocalPath + "\\" + Name;

            FileInfo _File = new FileInfo(_fileName);
            FileStream _FileStream;

            _FtpRequest = (FtpWebRequest)WebRequest.Create("ftp://" + _Host + Path + Name);

            _FtpRequest.Credentials = new NetworkCredential(_UserName, _Password);
            _FtpRequest.Method = WebRequestMethods.Ftp.DownloadFile;

            _FtpRequest.EnableSsl = _UseSSL;


            if (_File.Exists)
            {
                _FtpRequest.ContentOffset = _File.Length;
                _FileStream = new FileStream(_File.FullName, FileMode.Append, FileAccess.Write);
            }
            else
            {
                _FileStream = new FileStream(_File.FullName, FileMode.Create, FileAccess.Write);
            }
            
            _FtpResponse = (FtpWebResponse)_FtpRequest.GetResponse();

            Stream _ResponseStream = _FtpResponse.GetResponseStream();

            byte[] buffer = new byte[1024];
            int bytesRead = _ResponseStream.Read(buffer, 0, 1024);

            while (bytesRead != 0)
            {
                _FileStream.Write(buffer, 0, bytesRead);
                bytesRead = _ResponseStream.Read(buffer, 0, 1024);
            }

            _FileStream.Close();
            _ResponseStream.Close();
        }

    }
}
